/**
 *    \file    dice/src/be/l4/x0/L4X0BEWaitAnyFunction.cpp
 *  \brief   contains the implementation of the class CL4X0BEWaitAnyFunction
 *
 *    \date    06/01/2002
 *    \author  Ronald Aigner <ra3@os.inf.tu-dresden.de>
 */
/* Copyright (C) 2001-2004
 * Dresden University of Technology, Operating Systems Research Group
 *
 * This file contains free software, you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, Version 2 as
 * published by the Free Software Foundation (see the file COPYING).
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * For different licensing schemes please contact
 * <contact@os.inf.tu-dresden.de>.
 */

#include "L4X0BEWaitAnyFunction.h"
#include "be/l4/v2/L4V2BEIPC.h"
#include "be/BEFile.h"
#include "be/BEContext.h"
#include "be/BETrace.h"
#include "TypeSpec-Type.h"
#include <cassert>
using namespace std;

CL4X0BEWaitAnyFunction::CL4X0BEWaitAnyFunction(bool bOpenWait, bool bReply)
 : CL4BEWaitAnyFunction(bOpenWait, bReply)
{
}

/** destroys the wait function object */
CL4X0BEWaitAnyFunction::~CL4X0BEWaitAnyFunction()
{
}

/** \brief writes the unmarshalling code
 *  \param pFile the file to write to
 */
void
CL4X0BEWaitAnyFunction::WriteUnmarshalling(CBEFile* pFile)
{
    assert (m_pTrace);
    bool bLocalTrace = false;
    if (!m_bTraceOn)
    {
	m_pTrace->BeforeUnmarshalling(pFile, this);
	m_bTraceOn = bLocalTrace = true;
    }
    
    if (m_bReply)
        WriteMarshalReturn(pFile, false);
    else
        CL4BEWaitAnyFunction::WriteUnmarshalling(pFile);

    if (bLocalTrace)
    {
	m_pTrace->AfterUnmarshalling(pFile, this);
	m_bTraceOn = false;
    }
}
