/*
 * Misc. file.
 *
 * This file also contains code not suited anywhere else, it will get bigger
 * and needs to be split up then.
 *
 * $Id$
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>

#ifdef CONFIG_BLK_DEV_INITRD
#include <linux/initrd.h>
#endif

#include <linux/kprobes.h>

#include <asm/page.h>
#include <asm/setup.h>
#include <asm/uaccess.h>
#include <asm/segment.h>
#include <asm-generic/sections.h>

#include <l4/bid_config.h>
#include <l4/dm_phys/dm_phys.h>
#include <l4/dm_generic/types.h>
#include <l4/env/env.h>
#include <l4/env/errno.h>
#include <l4/generic_fprov/generic_fprov-client.h>
#include <l4/generic_io/libio.h>
#include <l4/generic_ts/generic_ts.h>
#include <l4/log/l4log.h>
#include <l4/log/log_printf.h>
#include <l4/semaphore/semaphore.h>
#include <l4/sys/kdebug.h>
#include <l4/sys/syscalls.h>
#include <l4/sys/utcb.h>
#include <l4/util/cpu.h>
#include <l4/sigma0/kip.h>
#include <l4/sigma0/sigma0.h>
#include <l4/util/mbi_argv.h>
#include <l4/util/l4_macros.h>

#include <asm/api/config.h>
#include <asm/api/macros.h>

#include <asm/l4lxapi/thread.h>
#include <asm/l4lxapi/misc.h>
#include <asm/l4lxapi/task.h>

#include <asm/generic/dispatch.h>
#include <asm/generic/ferret.h>
#include <asm/generic/upage.h>
#include <asm/generic/memory.h>
#include <asm/generic/setup.h>
#include <asm/generic/stack_id.h>
#include <asm/generic/stats.h>
#include <asm/generic/tamed.h>
#include <asm/generic/task.h> /* for l4x_id2task */

#include <asm/l4x/iodb.h>
#include <asm/l4x/exception.h>

#include <l4xi-server.h>
#include <l4xi-client.h>

#ifdef ARCH_x86
struct desc_struct cpu_gdt_table[NR_CPUS][GDT_ENTRIES];

unsigned l4x_fiasco_gdt_entry_offset;
#endif
#ifdef ARCH_arm
unsigned long cr_alignment;
#endif

unsigned l4x_fiasco_nr_of_syscalls = 7;

char LOG_tag[9] = "l4lx";

/* Configure place for the L4RM heap, put it above our main memory
 * dataspace, so that the address space below is small.
 */
const l4_addr_t l4rm_heap_start_addr = TASK_SIZE - (4 << 20);

/* Also put thread data at the end of the address space */
const l4_addr_t l4thread_stack_area_addr = TASK_SIZE - (256 << 20);
const l4_addr_t	l4thread_tcb_table_addr  = TASK_SIZE - (260 << 20);

extern asmlinkage void start_kernel(void);

l4_threadid_t linux_server_thread_id = L4_NIL_ID;
l4_threadid_t l4x_start_thread_id = L4_NIL_ID;
l4_threadid_t l4x_start_thread_pager_id = L4_NIL_ID;

l4dm_dataspace_t l4env_ds_mainmem;
l4dm_dataspace_t l4x_ds_isa_dma;
static void *l4x_main_memory_start;
static void *l4x_isa_dma_memory_start;
l4_uint32_t l4env_vmalloc_areaid;
unsigned long l4env_vmalloc_memory_start;
l4env_infopage_t *l4env_infopage;
l4_kernel_info_t *l4lx_kinfo;
unsigned int l4x_kernel_taskno;

#ifdef CONFIG_L4_DEBUG_STATS
long l4x_debug_stats_suspend;
long l4x_debug_stats_pagefault;
long l4x_debug_stats_exceptions;
#endif

typedef struct {
	l4_addr_t phys; /* physical address */
	void    * virt; /* virtual address */
	l4_size_t size; /* size of chunk in Bytes */
} l4env_phys_virt_mem_t;

#define L4ENV_PHYS_VIRT_ADDRS_MAX_ITEMS 10

/* Default memory size */
unsigned long l4env_mainmem_size = CONFIG_L4_L4ENV_MEMSIZE << 20;
const unsigned long ISA_DMA_SIZE = 2 << 20;

/* Set max amount of threads which can be created, weak symbol from
 * thread library */
const int l4thread_max_threads = 128;

static l4env_phys_virt_mem_t l4env_phys_virt_addrs[L4ENV_PHYS_VIRT_ADDRS_MAX_ITEMS];
int l4env_phys_virt_addr_items;

static const unsigned long required_kernel_abi_version = 5;
static const char *required_kernel_features[] =
  { "exception_ipc",
    "pagerexregs",
#ifdef CONFIG_L4_TAMED
    "deceit_bit_disables_switch",
#endif
#ifdef ARCH_x86
    "segments",
#endif
  };

void __init l4env_v2p_init(void)
{
	l4env_phys_virt_addr_items = 0;
}

void l4env_v2p_add_item(l4_addr_t phys, void *virt, l4_size_t size)
{
	if (l4env_phys_virt_addr_items == L4ENV_PHYS_VIRT_ADDRS_MAX_ITEMS)
		panic("v2p filled up!");

	l4env_phys_virt_addrs[l4env_phys_virt_addr_items++]
		= (l4env_phys_virt_mem_t){.phys = phys, .virt = virt, .size = size};
}

unsigned long l4env_virt_to_phys(volatile void * address)
{
	int i;

	for (i = 0; i < l4env_phys_virt_addr_items; i++) {
		if (l4env_phys_virt_addrs[i].virt <= address &&
		    address < l4env_phys_virt_addrs[i].virt
		              + l4env_phys_virt_addrs[i].size) {
			return (address - l4env_phys_virt_addrs[i].virt)
			       + l4env_phys_virt_addrs[i].phys;
		}
	}

	/* Whitelist: */

	/* Debugging check: don't miss a translation, can give nasty
	 *                  DMA problems */
	LOG_printf("%s: Could not translate virt. address %p\n",
	           __func__, address);
	enter_kdebug("Check this!");

	return __pa(address);
}

EXPORT_SYMBOL(l4env_virt_to_phys);


void *l4env_phys_to_virt(unsigned long address)
{
	int i;

	for (i = 0; i < l4env_phys_virt_addr_items; i++) {
		if (l4env_phys_virt_addrs[i].phys <= address &&
		    address < l4env_phys_virt_addrs[i].phys
		              + l4env_phys_virt_addrs[i].size) {
			return (address - l4env_phys_virt_addrs[i].phys)
			       + l4env_phys_virt_addrs[i].virt;
		}
	}

	/* Whitelist: */
	if (address < 0x1000 ||
	    address == 0xb8000 ||
	    address == 0xa0000 ||
	    address == 0xc0000)
		return __va(address);

	/* Debugging check: don't miss a translation, can give nasty
	 *                  DMA problems */
	LOG_printf("%s: Could not translate phys. address 0x%lx\n",
	           __func__, address);
	enter_kdebug("Check this!");

	return __va(address);
}

EXPORT_SYMBOL(l4env_phys_to_virt);


/* ----------------------------------------------------- */
/* Needed for external stuff (also for dice generated code) */
/* Do not use kmalloc here, first because kmalloc may use current and
 * the stacks of external libraries are not Linux conformable (although
 * GFP_ATOMIC may solve this), secondly kmalloc only supports up to 128k
 * allocations, which may not be enough for a generic malloc.
 * Downside of using l4dm_mem_allocatate: granularity of pages, i.e. wastes
 * memory, I need to get malloc on DSs implementation from somewhere... */
void *malloc(unsigned long size)
{
	return l4dm_mem_allocate(size, 0);
}

void free(void *ptr)
{
	l4dm_mem_release(ptr);
}

unsigned long strtoul(const char *s, char **ep, int base)
{
	return simple_strtoul(s, ep, base);
}


static void l4x_forward_pf(l4_umword_t addr, l4_umword_t pc)
{
	int error;
	l4_umword_t dw0, dw1;
	l4_msgdope_t result;

	do {
		error = l4_ipc_call(l4x_start_thread_pager_id,
		                    L4_IPC_SHORT_MSG, addr, pc,
				    L4_IPC_MAPMSG(addr, L4_LOG2_PAGESIZE),
				    &dw0, &dw1, L4_IPC_NEVER, &result);
	} while (error == L4_IPC_SECANCELED || error == L4_IPC_SEABORTED);

	if (unlikely(error))
		LOG_printf("Error forwarding page fault: %d\n", error);
}


/* To get mmap of /dev/mem working, map address space before
 * start of mainmem with a ro page,
 * lets try with this... */
static void l4x_mbm_request_ghost(l4dm_dataspace_t *ghost_ds)
{
	unsigned int i;
	void *addr;
	int ret;
	char page_name[15];
	static unsigned int page_nr = 0;

	snprintf(page_name, sizeof(page_name), "Ghost page %d", ++page_nr);

	/* Get a page from our dataspace manager */
	if ((ret = l4dm_mem_open(L4DM_DEFAULT_DSM, L4_PAGESIZE, L4_PAGESIZE,
				 L4DM_PINNED,
				 page_name, ghost_ds))) {
		LOG_printf("%s: Can't get ghost page: %s(%d)!\n",
			   __func__, l4env_errstr(ret), ret);
		l4x_exit_l4linux();
	}

	/* Map page RW */
	if ((ret = l4rm_attach(ghost_ds, L4_PAGESIZE, 0,
	                       L4DM_RW | L4RM_MAP, &addr))) {
		LOG_printf("%s: Can't map ghost page: %s(%d)\n",
		           __func__, l4env_errstr(ret), ret);
		l4x_exit_l4linux();
	}

	/* Write a certain value in to the page so that we can
	 * easily recognize it */
	for (i = 0; i < L4_PAGESIZE; i += sizeof(i))
		*(unsigned long *)((unsigned long)addr + i) = 0xcafeface;

	/* Detach it again */
	if ((ret = l4rm_detach(addr))) {
		LOG_printf("%s: Can't unmap ghost page: %s(%d)\n",
		           __func__, l4env_errstr(ret), ret);
		l4x_exit_l4linux();
	}

}

static void l4x_map_below_mainmem(void)
{
	unsigned long i;
	l4dm_dataspace_t ds, ghost_ds;
	l4_threadid_t dthr;
	l4_offs_t off;
	l4_addr_t map_addr;
	l4_size_t map_size;
	int ret;
	unsigned long i_inc;
	int map_count = 0, map_count_all = 0;

	LOG_printf("Filling lower ptabs...\n");
	LOG_flush();

	/* Loop through free address space before mainmem */
	for (i = L4_PAGESIZE; i < (unsigned long)l4x_main_memory_start; i += i_inc) {
		ret = l4rm_lookup((void *)i, &map_addr, &map_size,
		                  &ds, &off, &dthr);
		if (ret > 0) {
			// success, something there
			if (i != map_addr)
				enter_kdebug("shouldn't be, hmm?");
			i_inc = map_size;
			continue;
		}
		i_inc = L4_PAGESIZE;

		if (ret != -L4_ENOTFOUND) {
			LOG_printf("l4rm_lookup call failure: %s(%d)\n",
			           l4env_errstr(ret), ret);
			l4x_exit_l4linux();
		}

		if (!map_count) {
			/* Get new ghost page every 1024 mappings
			 * to overcome a Fiasco mapping db
			 * limitation. */
			l4x_mbm_request_ghost(&ghost_ds);
			map_count = 1014;
		}
		map_count--;
		map_count_all++;
		ret = l4rm_attach_to_region(&ghost_ds, (void *)i,
		                            L4_PAGESIZE, 0, L4DM_RO | L4RM_MAP);
		if (ret) {
			LOG_printf("%s: Can't attach to ghost page: %s(%d)!\n",
			           __func__, l4env_errstr(ret), ret);
			l4x_exit_l4linux();
		}
	}
	LOG_printf("Done (%d entries).\n", map_count_all);
	LOG_flush();

	/* Touch page so that we get the PF now */
	for (i = L4_PAGESIZE; i < (unsigned long)l4x_main_memory_start;
	     i += L4_PAGESIZE)
		l4x_forward_pf(i, 0);
}

/* map upage to UPAGE_USER_ADDRESS in linux server itself */
static void l4x_map_upage_myself(void)
{
	int ret;
	l4dm_dataspace_t ds;
	l4_offs_t off;
	l4_addr_t map_addr;
	l4_size_t map_size;
	l4_threadid_t dthr;

	if ((ret = l4rm_lookup((void *)&_upage_start,
		               &map_addr, &map_size, &ds, &off, &dthr))
	    != L4RM_REGION_DATASPACE) {
		LOG_printf("Cannot get dataspace of upage (%s(%d))",
		           l4env_errstr(ret), ret);
		enter_kdebug("get ds of upage");
	}

	if ((ret = l4rm_attach_to_region(&ds, (void *)UPAGE_USER_ADDRESS,
	                                 L4_PAGESIZE, off,
	                                 L4DM_RO | L4RM_MAP))) {
		LOG_printf("Cannot attach upage properly: (%s(%d))",
		           l4env_errstr(ret), ret);
		enter_kdebug("attach upage");
	}
}

void __init l4env_register_region(void *start, l4_size_t size, int allow_noncontig, const char *tag)
{
	const int num_addrs = 6;
	l4dm_mem_addr_t phys_addrs[num_addrs];
	l4_size_t phys_addrs_size;
	l4_offs_t offset;
	int pas, i;

	LOG_printf("%15s: virt: %p to %p [%u KiB]\n",
	           tag, start, start + size - 1, size >> 10);

	if ((pas = l4dm_mem_phys_addr(start, size,
	                              phys_addrs, num_addrs,
	                              &phys_addrs_size)) < 0) {
		LOG_printf("Error getting physical addresses for \"%s\" "
		           "(err = %s(%d)!", tag, l4env_errstr(pas), pas);
		l4x_exit_l4linux();
	}

	/* Debugging msgs, handle this carefully */
	if (!allow_noncontig && pas > 1) {
		LOG_printf("Noncontiguous region for %s\n", tag);
		/* enter_kdebug("Noncontinuous region!"); */
	}
	if (pas == num_addrs)
		LOG_printf("Probably region overflow for %s?!\n", tag);


	LOG_printf("%15s: Number of physical regions: %d, %u Bytes\n",
	           tag, pas, phys_addrs_size);

	offset = 0;
	for (i = 0; i < pas; i++) {
		l4env_v2p_add_item(phys_addrs[i].addr,
		                   start + offset,
		                   phys_addrs[i].size);

		LOG_printf("%15s: %d: Phys: 0x%08lx to 0x%08lx, Size: %8u\n",
		           tag, i+1, phys_addrs[i].addr,
		           phys_addrs[i].addr + phys_addrs[i].size,
		           phys_addrs[i].size);
		//LOG_printf("%s: Offset to virtual: 0x%08x\n", tag,
		//	   phys_addrs[i].addr - (unsigned long)start);
		offset += phys_addrs[i].size;
	}
}

/*
 * Register program section(s) for virt_to_phys, at least initdata is used
 * as normal storage later (including DMA usage).
 *
 * Note: we just register the dataspace region where __init_begin is in for
 *       now
 * Note2:
 */
static void __init l4env_register_pointer_section(void *p_in_addr,
                                                  int allow_noncontig,
                                                  const char *tag)
{
	l4dm_dataspace_t ds;
	l4_offs_t off;
	l4_addr_t addr;
	l4_size_t size;
	l4_threadid_t dthr;
	int res;

	if ((res = l4rm_lookup(p_in_addr, &addr, &size, &ds, &off, &dthr))
	    != L4RM_REGION_DATASPACE) {
		LOG_printf("Cannot find dataspace at %p?!"
		           "(err = %s(%d))\n", p_in_addr,
		           l4env_errstr(res), res);
		enter_kdebug("l4rm_lookup failed");
		return;
	}

	LOG_printf("%s: addr = %08lx size = %d\n", __func__, addr, size);

	l4env_register_region((void *)addr, size, allow_noncontig, tag);
}

void __init setup_l4env_memory(char *cmdl,
                               unsigned long *main_mem_start,
                               unsigned long *main_mem_size,
                               unsigned long *isa_dma_mem_start,
                               unsigned long *isa_dma_mem_size)
{
	int res;
	unsigned long memory_area_size;
	l4_uint32_t memory_area_id;
	l4_addr_t memory_area_addr;
	l4_size_t poolsize, poolfree;
#ifdef ARCH_x86
	extern unsigned long init_pg_tables_end;
#endif
	l4_uint32_t dm_flags = L4DM_CONTIGUOUS | L4DM_PINNED;
	l4_uint32_t rm_flags = L4DM_RW;

	/* See if we find a mem=xxx option in the command line */
	if ((cmdl = strstr(cmdl, "mem=")) && (res = memparse(cmdl+4, &cmdl)))
		l4env_mainmem_size = res;

	/* Make sure that main memory starts at a new superpage. If dm_phys
	 * (accidently) allocates our memory starting at a superpage in his
	 * address space, we get L4_SUPERPAGE_SIZE mappings. */
	rm_flags |= L4RM_SUPERPAGE_ALIGNED;

	if ((l4env_mainmem_size % L4_SUPERPAGESIZE) == 0) {
		LOG_printf("%s: Forcing superpages for main memory\n", __func__);
		/* force dm_phys to allocate superpages */
		dm_flags |= L4DM_MEMPHYS_SUPERPAGES;
		rm_flags |= L4DM_MEMPHYS_SUPERPAGES;
	}

	/* Allocate main memory */
	if ((res = l4dm_mem_open(L4DM_DEFAULT_DSM, l4env_mainmem_size,
	                         L4_PAGESIZE, dm_flags,
	                         "L4Linux main memory", &l4env_ds_mainmem))) {
		LOG_printf("%s: Can't get main memory of %ldMB: %s(%d)!\n",
		           __func__, l4env_mainmem_size >> 20,
		           l4env_errstr(res), res);
		l4x_exit_l4linux();
	}

	/* if there's a '+' at the end of the mem=-option try to get
	 * more memory */
	if (cmdl && *cmdl == '+') {
		unsigned long chunksize = 64 << 20;

		while (chunksize > (4 << 20)) {
			if ((res = l4dm_mem_resize(&l4env_ds_mainmem,
						   l4env_mainmem_size
			                           + chunksize)))
				chunksize >>= 1; /* failed */
			else
				l4env_mainmem_size += chunksize;
		}
	}

	LOG_printf("Main memory size: %ldMB\n", l4env_mainmem_size >> 20);
	if (l4env_mainmem_size < (4 << 20)) {
		LOG_printf("Not enough main memory!\n");
		l4x_exit_l4linux();
	}

	memory_area_size = l4env_mainmem_size;

	/* Try to get ISA DMA memory */
	/** In the default config dm_phys prints out messages if
	 ** the allocation fails, so query the size separately to
	 ** not disturb users with error messages */
	if ((res = l4dm_memphys_poolsize(L4DM_MEMPHYS_ISA_DMA,
	                                 &poolsize, &poolfree))) {
		LOG_printf("Cannot query ISA DMA pool size: %s(%d)\n",
		           l4env_errstr(res), res);
		l4x_exit_l4linux();
	}
	if (poolfree >= ISA_DMA_SIZE
	    && !l4dm_memphys_open(L4DM_MEMPHYS_ISA_DMA,
	                          L4DM_MEMPHYS_ANY_ADDR,
	                          ISA_DMA_SIZE, L4_PAGESIZE,
	                          L4DM_CONTIGUOUS, "L4Linux ISA DMA memory",
	                          &l4x_ds_isa_dma)) {
		LOG_printf("Got %lukB of ISA DMA memory.\n",
		           ISA_DMA_SIZE >> 10);
		memory_area_size += l4_round_superpage(ISA_DMA_SIZE);
	} else
		l4x_ds_isa_dma = L4DM_INVALID_DATASPACE;

	/* Get contiguous region in our virtual address space to put
	 * the data spaces in */
	if (l4rm_area_reserve(memory_area_size, L4RM_SUPERPAGE_ALIGNED,
			      &memory_area_addr, &memory_area_id)) {
		LOG_printf("Error reserving memory area: %s(%d)\n",
		           l4env_errstr(res), res);
		l4x_exit_l4linux();
	}

	/* Attach data spaces to local address space */
	/** First: the ISA DMA memory */
	if (!l4dm_is_invalid_ds(l4x_ds_isa_dma)) {
		l4x_isa_dma_memory_start = (void *)memory_area_addr;
		l4x_main_memory_start =
			(void *)(l4x_isa_dma_memory_start
		                 + l4_round_superpage(ISA_DMA_SIZE));
		if ((res = l4rm_area_attach_to_region(&l4x_ds_isa_dma,
	                                              memory_area_id,
	                                              l4x_isa_dma_memory_start,
	                                              ISA_DMA_SIZE, 0,
	                                              L4DM_RW | L4RM_MAP))) {
			LOG_printf("Error attaching to ISA DMA memory: %s(%d)\n",
				   l4env_errstr(res), res);
			l4x_exit_l4linux();
		}
	} else
		l4x_main_memory_start = (void *)memory_area_addr;

	/** Second: the main memory */
	if ((res = l4rm_area_attach_to_region(&l4env_ds_mainmem,
	                                      memory_area_id,
	                                      l4x_main_memory_start,
	                                      l4env_mainmem_size, 0,
	                                      rm_flags | L4RM_MAP))) {
		LOG_printf("Error attaching to L4Linux main memory: %s(%d)\n",
		           l4env_errstr(res), res);
		l4x_exit_l4linux();
	}

	/* Release area ... make possible hole available again */
	if ((res = l4rm_area_release(memory_area_id))) {
		LOG_printf("Error releasing area: %s(%d)\n",
		           l4env_errstr(res), res);
		l4x_exit_l4linux();
	}

	*main_mem_start = (unsigned long)l4x_main_memory_start;
	*main_mem_size  = l4env_mainmem_size;

	if (l4dm_is_invalid_ds(l4x_ds_isa_dma))
		*isa_dma_mem_start = *isa_dma_mem_size = 0;
	else {
		*isa_dma_mem_start = (unsigned long)l4x_isa_dma_memory_start;
		*isa_dma_mem_size  = ISA_DMA_SIZE;
	}

#ifdef ARCH_x86
	init_pg_tables_end = memory_area_addr;
#endif

	if (!l4dm_is_invalid_ds(l4x_ds_isa_dma))
		l4env_register_region(l4x_isa_dma_memory_start, ISA_DMA_SIZE,
		                      0, "ISA DMA memory");
	l4env_register_region(l4x_main_memory_start, l4env_mainmem_size,
	                      0, "Main memory");

	/* Reserve some part of the virtual address space for vmalloc */
	if ((res = l4rm_area_reserve(
#ifdef ARCH_x86
	                             __VMALLOC_RESERVE,
#else
	                             VMALLOC_SIZE << 20,
#endif
	                             L4RM_LOG2_ALIGNED,
	                             (l4_addr_t *)&l4env_vmalloc_memory_start,
				     &l4env_vmalloc_areaid))) {
		LOG_printf("%s: Error reserving vmalloc memory: %s(%d)!\n",
		           __func__, l4env_errstr(res), res);
		l4x_exit_l4linux();
	}

	l4x_map_below_mainmem();
}

unsigned long l4x_get_isa_dma_memory_end(void)
{
	return (unsigned long)l4x_isa_dma_memory_start
	       + ISA_DMA_SIZE;
}

static void __init setup_stack(void)
{
	struct thread_info *ti = current_thread_info();

	*ti = (struct thread_info) INIT_THREAD_INFO(init_task);
	ti->task->thread_info = ti;

	l4x_stack_setup(ti);
}

/* Do not put this on the stack of its user */
static CORBA_Server_Environment _server_env = dice_default_server_environment;

/*
 * This is the panic blinking function, we misuse it to sleep forever.
 */
static long l4x_blink(long time)
{
	printk("panic: going to sleep forever, bye\n");
	LOG_printf("panic: going to sleep forever, bye\n");
	l4_sleep_forever();
	return 0;
}

static void __init l4env_linux_startup(void *data)
{
	l4_threadid_t caller_id = *(l4_threadid_t *)data;

	LOG_printf("%s thread %x.\n", __func__, l4_myself().id.lthread);

	/* Wait for start signal */
	l4xi_linux_main_startup_recv(&caller_id, &_server_env);

	LOG_printf("main thread: received startup message.\n");

	linux_server_thread_id = l4_myself();

	l4lx_thread_pager_change(linux_server_thread_id, caller_id);

#ifdef ARCH_x86
	l4_utcb_get()->status |= L4_UTCB_EXCEPTION_IPC_ENABLED;
	if (sizeof(l4_utcb_t) != 128)
		enter_kdebug("Weird UTCB size");
#endif
	setup_stack();

	panic_blink = l4x_blink;

	start_kernel();
}

#ifdef ARCH_x86
void __init l4x_l4io_init(void)
{
	l4io_info_t *io_info_page = (void *)-1;

	/* initialize IO lib */
	if (l4io_init(&io_info_page, L4IO_DRV_INVALID))
		enter_kdebug("Error calling l4io_init!");
}
#endif

#ifdef CONFIG_BLK_DEV_INITRD
static l4dm_dataspace_t l4env_initrd_ds;
static unsigned long l4env_initrd_mem_start;

/**
 * Get the RAM disk from the file provider!
 *
 * \param filename    File name in the form {(nd)}/path/to/file
 * \param *rd_start   Start of ramdisk
 * \param *rd_end     End of ramdisk
 *
 * \return 0 on succes, != 0 otherwise
 */
static int __init fprov_load_initrd(const char *filename,
                                    unsigned long *rd_start,
                                    unsigned long *rd_end)
{
	int error;
	l4_size_t size;
	CORBA_Environment env = dice_default_environment;

	if (l4_thread_equal(l4env_infopage->fprov_id, L4_INVALID_ID)) {
		LOG_printf("File provider not set!\n");
		enter_kdebug("ATT!");
		return 1;
	}

	/* load RAM disk to DS */
	if ((error = l4fprov_file_open_call
	              (&l4env_infopage->fprov_id,
	               filename,
	               &l4env_infopage->memserv_id,
	               L4DM_CONTIGUOUS,
	               &l4env_initrd_ds,
	               &size,
	               &env))) {
		LOG_flush();
		LOG_printf("Couldn't load RAM disk from fprov!\n");
		enter_kdebug("ATT");
		return error;
	}
	LOG_printf("INITRD: Size of RAMdisk is %dKiB\n", size >> 10);
	LOG_flush();

	/* attach DS to region mapper */
	if ((error = l4rm_attach(&l4env_initrd_ds,
	                         size,
	                         0,
	                         L4DM_RW,
	                         (void **)rd_start))) {
		LOG_flush();
		LOG_printf("Couldn't attach RAM disk memory! err=%d\n", error);
		enter_kdebug("ATT");
		return error;
	}

	l4env_initrd_mem_start = *rd_start;
	*rd_end = *rd_start + size;
	printk("INITRD: %08lx - %08lx\n", *rd_start, *rd_end);

	if (size * 2 > l4env_mainmem_size) {
		LOG_printf("WARNING: RAMdisk size of %dMB probably too big\n"
		           "for %ldMB main memory. Sleeping a bit...\n",
		           size >> 20, l4env_mainmem_size >> 20);
		l4_sleep(20000);
	}

	LOG_flush();

	return 0;
}

/**
 * Free InitRD memory.
 */
void l4env_free_initrd_mem(void)
{
	printk("INITRD: Freeing memory.\n"); LOG_flush();

	/* detach memory */
	if (l4rm_detach((void *)l4env_initrd_mem_start))
		enter_kdebug("Error detaching from initrd mem!");

	/* close DS and free mem */
	if (l4dm_close(&l4env_initrd_ds))
		enter_kdebug("Error closing initrd ds!");
}


#define L4ENV_MAX_RD_PATH 200
static char l4env_rd_path[L4ENV_MAX_RD_PATH];

void __init l4env_load_initrd(char *command_line)
{
	char *sa, *se;
	char param_str[] = "l4env_rd=";
	int i, b;

	sa = command_line;
	while (*sa) {
		for (i = 0, b = 1; param_str[i] && b; i++)
			b = sa[i] == param_str[i];
		if (b)
			break;
		sa++;
	}
	if (*sa) {
		sa += strlen(param_str);
		se = sa;

		while (*se && *se != ' ')
			se++;
		if (se - sa > L4ENV_MAX_RD_PATH) {
			enter_kdebug("l4env_rd_path > L4ENV_MAX_RD_PATH");
			return;
		}
		strncpy(l4env_rd_path, sa, se - sa);
		LOG_printf("l4env_rd_path: %s\n", l4env_rd_path);
	}

	if (l4env_rd_path && *l4env_rd_path) {
		LOG_printf("Loading: %s\n", l4env_rd_path);

		if (fprov_load_initrd(l4env_rd_path,
		                      &initrd_start,
		                      &initrd_end)) {
			LOG_flush();
			LOG_printf("Couldn't load ramdisk :(\n");
			return;
		}

		initrd_below_start_ok = 1;

		//reserve_bootmem(initrd_start, initrd_end - initrd_start);

		LOG_printf("RAMdisk from %08lx to %08lx [%ldKiB]\n",
		           initrd_start, initrd_end,
		           (initrd_end - initrd_start) >> 10);
	}
}
#endif /* CONFIG_BLK_DEV_INITRD */

static void __init get_initial_cpu_capabilities(void)
{
#ifdef ARCH_x86
	/* new_cpu_data is filled into boot_cpu_data in setup_arch */
	extern struct cpuinfo_x86 new_cpu_data;
#ifdef CONFIG_L4_USERPRIV_ONLY
	/* The check uses popfl/pushfl and we don't like this with UX, and
	 * additionally nobody is going to use the UX version on a 486 type
	 * machine, right? And we _could_ also set the capabilities to 0... */
	new_cpu_data.x86_capability[0] = l4util_cpu_capabilities_nocheck();
#else
	new_cpu_data.x86_capability[0] = l4util_cpu_capabilities();
#endif
#endif
}

int main(int argc, char **argv)
{
	CORBA_Environment env = dice_default_environment;
	CORBA_Server_Environment server_env = {
	    .malloc = malloc,
	    .free   = free,
	    .timeout = L4_IPC_NEVER,
	};

	l4_threadid_t main_id;
	extern char _end[];
	extern char saved_command_line[];
	unsigned i;
	char *p;

	LOG_printf("\033[34;1m======> L4Linux 2.6 starting... <========\033[0m\n");
	LOG_printf("%s", linux_banner);
	LOG_printf("Binary name: %s\n", (*argv)?*argv:"NULL??");

	argv++;
	p = saved_command_line;
	while (*argv) {
		i = strlen(*argv);
		if (p - saved_command_line + i >= COMMAND_LINE_SIZE) {
			LOG_printf("Command line too long!");
			enter_kdebug("Command line too long!");
		}
		strcpy(p, *argv);
		p += i;
		if (*++argv)
			*p++ = ' ';
	}
	LOG_printf("Kernel command line (%d args): %s\n",
	           argc - 1, saved_command_line);

	l4lx_kinfo = l4sigma0_kip_map(L4_INVALID_ID);

	for (i = 0; i < sizeof(required_kernel_features)
		          / sizeof(required_kernel_features[0]); i++) {
		if (!l4sigma0_kip_kernel_has_feature(required_kernel_features[i])) {
			LOG_printf("The running kernel does not have the\n"
			           "      %s\n"
			           "feature enabled!\n",
			           required_kernel_features[i]);
			enter_kdebug("kernel feature missing!");
		}
	}

	if (l4sigma0_kip_kernel_abi_version() < required_kernel_abi_version) {
		LOG_printf("The kernel ABI version is too low: kernel has %ld, "
		           "I need %ld\n",
		           l4sigma0_kip_kernel_abi_version(),
		           required_kernel_abi_version);
		enter_kdebug("Stop!");
	}

	if (l4sigma0_kip_kernel_has_feature("pl0_hack"))
		l4x_fiasco_nr_of_syscalls++;

	if ((l4env_infopage = l4env_get_infopage()) == NULL) {
		LOG_printf("Couldn't get L4Env info page!\n");
		enter_kdebug("Stop!");
	}

	LOG_printf("Image: %08lx - %08lx [%u KiB].\n",
	           (unsigned long)_stext, (unsigned long)_end,
	           (_end - _stext + 1) >> 10);

	LOG_printf("Areas: Text:     %08lx - %08lx [%ldkB] (a bit longer)\n",
	           (unsigned long)&_stext, (unsigned long)&_sdata,
	           ((unsigned long)&_sdata - (unsigned long)&_stext) >> 10);
	LOG_printf("       Data:     %08lx - %08lx [%ldkB]\n",
	           (unsigned long)&_sdata, (unsigned long)&_edata,
	           ((unsigned long)&_edata - (unsigned long)&_sdata) >> 10);
	LOG_printf("       Initdata: %08lx - %08lx [%ldkB]\n",
	           (unsigned long)&__init_begin, (unsigned long)&__init_end,
	           ((unsigned long)&__init_end - (unsigned long)&__init_begin) >> 10);
	LOG_printf("       BSS:      %08lx - %08lx [%ldkB]\n",
	           (unsigned long)&__bss_start, (unsigned long)&__bss_stop,
	           ((unsigned long)&__bss_stop - (unsigned long)&__bss_start) >> 10);

	/* Touch areas to avoid pagefaults
	 * Data needs to be touched rw, code ro
	 */
	l4_touch_ro(&_stext, (unsigned long)&_sdata - (unsigned long)&_stext);
	l4_touch_rw(&_sdata, (unsigned long)&_edata - (unsigned long)&_sdata);
	l4_touch_ro(&__init_begin,
	            (unsigned long)&__init_end - (unsigned long)&__init_begin);
	l4_touch_rw(&__bss_start,
	            (unsigned long)&__bss_stop - (unsigned long)&__bss_start);

#ifdef CONFIG_L4_DROPSCON
	if (0) {
		extern void dropscon_bootlog_init(void);
		dropscon_bootlog_init();
	}
#endif

	/* some things from head.S */
	get_initial_cpu_capabilities();

	/* This is not the final try to set linux_server_thread_id
	 * but l4lx_thread_create needs some initial data for the return
	 * value... */
	linux_server_thread_id = l4_myself();

	/* Init v2p list */
	l4env_v2p_init();

	l4x_start_thread_id = l4_myself();
	l4x_start_thread_pager_id = l4_thread_ex_regs_pager(l4x_start_thread_id);
	l4x_kernel_taskno = l4x_start_thread_id.id.task;

#ifdef CONFIG_L4_TAMED
	l4x_tamed_init();
#endif

#ifdef ARCH_x86
	if (l4sigma0_kip_kernel_has_feature("io_prot")) {
#ifndef CONFIG_L4_TAMED
		enter_kdebug("ioprot-kernel -> enable TAMED mode!");
#endif
		l4x_iodb_init();
	}

	/* Initialize GDT entry offset */
	l4x_fiasco_gdt_entry_offset = fiasco_gdt_get_entry_offset();

	l4_utcb_get()->status  |= L4_UTCB_EXCEPTION_IPC_ENABLED;
	l4_utcb_get()->rcv_size = L4_UTCB_EXCEPTION_REGS_SIZE;

#ifdef CONFIG_L4_FERRET
	l4x_ferret_init();
#endif

#endif

	/* Set name of startup thread */
	l4lx_thread_name_set(l4x_start_thread_id, "l4env-start");

	/* fire up Linux server, will wait until start message */
	main_id = l4lx_thread_create(l4env_linux_startup,
	                             (char *)init_stack + sizeof(init_stack),
	                             &l4x_start_thread_id,
	                             sizeof(l4x_start_thread_id),
	                             CONFIG_L4_PRIO_SERVER,
	                             "server");

	LOG_printf("main thread will be " PRINTF_L4TASK_FORM "\n",
	           PRINTF_L4TASK_ARG(main_id));

	l4env_register_pointer_section(&__init_begin, 0, "sec-w-init");

	/* The next are not necessary as no-one has done virt_to_phys
	 * for these areas so far */
	//l4env_register_pointer_section(&_text,  1, "text");
	//l4env_register_pointer_section(&_edata, 0, "data");

	/* We do this here in the startup thread as we only have the right
	 * to do it here, if we want to do that in the main thread, we'd
	 * have to first share it here. */
	l4x_map_upage_myself();

	/* Send start message to main thread. */
	l4xi_linux_main_startup_send(&main_id, &env);

	LOG_printf("Main thread running, waiting...\n");

	l4xi_server_loop(&server_env);

	return 0;
}

void
l4xi_linux_main_startup_component(CORBA_Object _dice_corba_obj,
                                  CORBA_Server_Environment *_dice_corba_env)
{
	LOG_printf("L4Linux startup.\n");
}

void
l4xi_linux_main_exit_component(CORBA_Object _dice_corba_obj,
                               CORBA_Server_Environment *_dice_corba_env)
{
	extern void exit(int);
	LOG_printf("Terminating L4Linux.\n");
	exit(0);
}

void
l4xi_large_enough_message_buffer_component(CORBA_Object _dice_corba_obj,
                                           unsigned long v1,
                                           unsigned long v2,
                                           char *s,
                                           CORBA_Server_Environment *_dice_corba_env)
{}


#ifdef ARCH_x86
static void l4x_setup_die_utcb(l4xi_msg_buffer_t *buf)
{
	struct pt_regs regs;
	unsigned long regs_addr;
	extern void die(const char *msg, struct pt_regs *regs, int err);
	char message[40];
	l4_utcb_t *utcb = l4_utcb_get();

	snprintf(message, sizeof(message), "Trap: %ld", utcb->exc.trapno);
	message[sizeof(message) - 1] = 0;

	utcb_to_ptregs(utcb, &regs);

	/* XXX: check stack boundaries, i.e. utcb->exc.esp & (THREAD_SIZE-1)
	 * >= THREAD_SIZE - sizeof(thread_struct) - sizeof(struct pt_regs)
	 * - ...)
	 */
	/* Copy pt_regs on the stack */
	utcb->exc.esp -= sizeof(struct pt_regs);
	*(struct pt_regs *)utcb->exc.esp = regs;
	regs_addr = utcb->exc.esp;

	/* Put arguments for die on stack */
	/* err */
	utcb->exc.esp -= sizeof(unsigned long);
	*(unsigned long *)utcb->exc.esp = utcb->exc.err;
	/* regs */
	utcb->exc.esp -= sizeof(unsigned long);
	*(unsigned long *)utcb->exc.esp = regs_addr;
	/* msg */
	utcb->exc.esp -= sizeof(unsigned long);
	*(unsigned long *)utcb->exc.esp = (unsigned long)message;

	utcb->exc.esp -= sizeof(unsigned long);
	*(unsigned long *)utcb->exc.esp = 0;
	/* Set PC to die function */
	utcb->exc.eip = (unsigned long)die;

	utcb->snd_size = L4_UTCB_EXCEPTION_REGS_SIZE;
}

static void l4x_do_intra_iret(struct pt_regs regs)
{
	asm volatile ("mov %%cs, %0" : "=r" (regs.xcs));
	asm volatile
	("movl %0, %%esp	\t\n"
	 "popl %%ebx		\t\n"
	 "popl %%ecx		\t\n"
	 "popl %%edx		\t\n"
	 "popl %%esi		\t\n"
	 "popl %%edi		\t\n"
	 "popl %%ebp		\t\n"
	 "popl %%eax		\t\n"
	 "addl $12, %%esp	\t\n" /* keep ds, es; skip orig_eax */
	 "iret			\t\n"
	 : : "r" (&regs));

	panic("Kprobes zombie walking!");
}

static void l4x_setup_stack_for_traps(l4_utcb_t *utcb, struct pt_regs *regs,
                                      fastcall void (*trap_func)(struct pt_regs *regs, long err))
{
	const int l4x_intra_regs_size
		= sizeof(struct pt_regs) - 2 * sizeof(unsigned long);

	/* Copy pt_regs on the stack but omit last to two dwords,
	 * an intra-priv exception/iret doesn't have those, and native
	 * seems to do it the same, with the hope that nobody touches
	 * there in the pt_regs */
	utcb->exc.esp -= l4x_intra_regs_size;
	memcpy((void *)utcb->exc.esp, regs, l4x_intra_regs_size);

	/* do_<exception> functions are fastcall, arguments go in regs */
	utcb->exc.eax = utcb->exc.esp;
	utcb->exc.ebx = utcb->exc.err;

	/* clear TF */
	utcb->exc.eflags &= ~256;

	utcb->exc.esp -= sizeof(unsigned long);
	*(unsigned long *)utcb->exc.esp = 0; /* Return of l4x_do_intra_iret */
	utcb->exc.esp -= sizeof(unsigned long);
	*(unsigned long *)utcb->exc.esp = (unsigned long)l4x_do_intra_iret;

	/* Set PC to trap function */
	utcb->exc.eip = (unsigned long)trap_func;

	utcb->snd_size = L4_UTCB_EXCEPTION_REGS_SIZE;
}

#ifdef CONFIG_KPROBES
static int l4x_handle_kprobes(l4xi_msg_buffer_t *buf)
{
	extern fastcall void do_int3(struct pt_regs *regs, long err);
	struct pt_regs regs;

	if (kprobe_running())
		return 1; /* Not handled */

	/* check for kprobes break instruction */
	if (*(unsigned char *)l4_utcb_exc_pc(l4_utcb_get()) != BREAKPOINT_INSTRUCTION)
		return 1; /* Not handled */

	utcb_to_ptregs(l4_utcb_get(), &regs);

	/* Set after breakpoint instruction as for HLT pc is on the
	 * instruction and for INT3 after the instruction */
	regs.eip++;

	l4x_setup_stack_for_traps(l4_utcb_get(), &regs, do_int3);
	return 0;
}
#else
static inline int l4x_handle_kprobes(l4xi_msg_buffer_t *buf)
{
  return 1; /* Not handled */
}
#endif

static int l4x_handle_int1(l4xi_msg_buffer_t *buf)
{
	struct pt_regs regs;
	extern fastcall void do_debug(struct pt_regs *regs, long err);

	if (l4_utcb_get()->exc.trapno != 1)
		return 1; /* Not handled */

	utcb_to_ptregs(l4_utcb_get(), &regs);
	l4x_setup_stack_for_traps(l4_utcb_get(), &regs, do_debug);
	return 0;
}

static int l4x_handle_clisti(l4xi_msg_buffer_t *buf)
{
	unsigned char opcode = *(unsigned char *)l4_utcb_exc_pc(l4_utcb_get());
	extern void exit(int);

	/* check for cli or sti instruction */
	if (opcode != 0xfa && opcode != 0xfb)
		return 1; /* not handled if not those instructions */

	/* If we trap those instructions it's most likely a configuration
	 * error and quite early in the boot-up phase, so just quit. */
	LOG_printf("Aborting L4Linux due to unexpected CLI/STI instructions.\n");
	exit(0);

	return 0;
}

#ifdef CONFIG_L4_DEBUG_SHOW_SERVER_EXCEPTIONS
static inline void l4x_print_exception(l4_threadid_t t, l4xi_msg_buffer_t *buf)
{
	LOG_printf("EX: "l4util_idfmt": pc = "l4_addr_fmt" trapno = 0x%lx err = 0x%lx\n",
	           l4util_idstr(t), l4_utcb_get()->exc.eip,
		   l4_utcb_get()->exc.trapno, l4_utcb_get()->exc.err);
}
#endif
#endif /* ARCH_x86 */

#ifdef ARCH_arm
static void l4x_setup_die_utcb(l4xi_msg_buffer_t *buf)
{
	struct pt_regs regs;
	extern void die(const char *msg, struct pt_regs *regs, int err);
	char message[40];
	l4_utcb_t __utcb;
	l4_utcb_t *utcb = &__utcb;

	snprintf(message, sizeof(message), "Boom!");
	message[sizeof(message) - 1] = 0;

	*utcb = *(l4_utcb_t *)(DICE_GET_STRING(buf, 0));

	utcb_to_ptregs(utcb, &regs);

	/* Copy pt_regs on the stack */
	utcb->exc.sp -= sizeof(struct pt_regs);
	*(struct pt_regs *)utcb->exc.sp = regs;

	/* Put arguments for die into registers */
	utcb->exc.r[0] = (unsigned long)message;
	utcb->exc.r[1] = utcb->exc.sp;
	utcb->exc.r[2] = utcb->exc.err;

	/* Set PC to die function */
	utcb->exc.pc  = (unsigned long)die;
	utcb->exc.ulr = 0;

	DICE_SIZE_DOPE(buf) = L4_IPC_DOPE(3, 1);
	DICE_SEND_DOPE(buf) = L4_IPC_DOPE(3, 1);
	DICE_MARSHAL_STRING(buf, DICE_GET_STRING(buf, 0),
	                    sizeof(l4_utcb_t), 0);
	*(l4_utcb_t *)(DICE_GET_STRING(buf, 0)) = *utcb;
}

static inline int l4x_handle_kprobes(l4xi_msg_buffer_t *buf)
{
	return 1; /* Not handled */
}

static inline int l4x_handle_int1(l4xi_msg_buffer_t *buf)
{
	return 1; /* Not handled */
}

static inline int l4x_handle_clisti(l4xi_msg_buffer_t *buf)
{
	return 1; /* Not handled */
}

static inline int l4_is_io_page_fault(l4_umword_t d)
{
	return 0;
}

#ifdef CONFIG_L4_DEBUG_SHOW_SERVER_EXCEPTIONS
static inline void l4x_print_exception(l4_threadid_t t, l4xi_msg_buffer_t *buf)
{
	l4_utcb_t __utcb;
	l4_utcb_t *utcb = &__utcb;

	*utcb = *(l4_utcb_t *)(DICE_GET_STRING(buf, 0));

	LOG_printf("EX: "l4util_idfmt": pc = "l4_addr_fmt" err = 0x%lx\n",
	           l4util_idstr(t), utcb->exc.pc, utcb->exc.err);
}
#endif
#endif /* ARCH_arm */

int l4xi_linux_default_handle(CORBA_Object src_id, l4xi_msg_buffer_t *buf,
                              CORBA_Server_Environment *env)
{
	l4_umword_t dw0, dw1;

	dw0 = DICE_GET_DWORD(buf, 0);
	dw1 = DICE_GET_DWORD(buf, 1);

	if (!l4_utcb_exc_is_exc_ipc(dw0, dw1)
	    && !l4_is_io_page_fault(dw0)) {
		static unsigned long old_pf_addr = ~0UL, old_pf_pc = ~0UL;
		if (unlikely(old_pf_addr == (dw0 & ~1) && old_pf_pc == dw1)) {
			LOG_printf("Double page fault dw0=%08lx dw1=%08lx\n",
			           dw0, dw1);
			enter_kdebug("Double pagefault");
		}
		old_pf_addr = dw0 & ~1;
		old_pf_pc   = dw1;
	}

	if (unlikely(!l4_thread_equal(*src_id, linux_server_thread_id))) {
		LOG_printf("Invalid source for request: "l4util_idfmt"\n",
		           l4util_idstr(*src_id));
		return DICE_NO_REPLY;
	}

	if (l4_utcb_exc_is_exc_ipc(dw0, dw1)) {

#ifdef CONFIG_L4_DEBUG_SHOW_SERVER_EXCEPTIONS
		l4x_print_exception(*src_id, buf);
#endif

		if (l4x_handle_kprobes(buf))
			if (l4x_handle_int1(buf))
				if (l4x_handle_clisti(buf))
					l4x_setup_die_utcb(buf);

		DICE_GET_DWORD(buf, 0) = 0;
		DICE_GET_DWORD(buf, 1) = 0;
		DICE_SET_SHORTIPC_COUNT(buf);
		return DICE_REPLY;
	}

#ifdef CONFIG_L4_DEBUG_SHOW_SERVER_EXCEPTIONS
	LOG_printf("PF: "l4util_idfmt": pfaddr = "l4_addr_fmt
	           " pc = "l4_addr_fmt" (%s%s)\n",
	           l4util_idstr(*src_id), dw0, dw1,
	           dw0 & 2 ? "rw" : "ro", dw0 & 1 ? ", T" : "");
#endif

#ifdef ARCH_x86
	/* If it's CLI/STI, come back with an exception */
	if (l4_is_io_page_fault(dw0)
	    && ((l4_fpage_t)(dw0)).iofp.iopage == 0
	    && ((l4_fpage_t)(dw0)).iofp.iosize == L4_WHOLE_IOADDRESS_SPACE) {
		DICE_GET_DWORD(buf, 0) = -1;
		DICE_SET_SHORTIPC_COUNT(buf);
		return DICE_REPLY;
	}
#endif

	/* For a 0 pointer deref, come back with an exception */
	if ((dw0 & ~3) == 0)
		DICE_GET_DWORD(buf, 0) = -1;
	else {
		/* Forward page fault to our pager */
		l4x_forward_pf(dw0, dw1);
		DICE_GET_DWORD(buf, 0) = 0;
	}

	DICE_GET_DWORD(buf, 1) = 0;
	DICE_SET_SHORTIPC_COUNT(buf);

	return DICE_REPLY;
}


void l4x_exit_l4linux(void)
{
	CORBA_Environment env = dice_default_environment;
	l4xi_linux_main_exit_send(&l4x_start_thread_id, &env);
}

int l4x_map_iomemory_from_sigma0(l4_addr_t phys, l4_addr_t virt, l4_size_t size)
{
	l4_threadid_t sigma0_id = L4_NIL_ID;
	int ret;

	sigma0_id.id.task    = 2;
	sigma0_id.id.lthread = 0;

	if ((ret = l4sigma0_map_iomem(sigma0_id, phys, virt, size, 0))) {
		printk("Error mapping IO memory from Sigma0. "
		       "Error: %d (%s)\n", ret, l4sigma0_map_errstr(ret));
		return 1;
	}

	return 0;
}

typedef void (*at_exit_function_type)(void);

static at_exit_function_type at_exit_functions[3];
static const int at_exit_nr_of_functions
   = sizeof(at_exit_functions) / sizeof(at_exit_functions[0]);

int atexit(void (*function)(void))
{
	int i;
	LOG_printf("atexit(function = %p) called\n", function);

	for (i = 0; i < at_exit_nr_of_functions; i++)
		if (!at_exit_functions[i]) {
			at_exit_functions[i] = function;
			return 0;
		}

	LOG_printf("%s: We only support %d atexit functions!\n",
	           __func__, at_exit_nr_of_functions);
	return 1;
}

static void atexit_run_functions(void)
{
	int i;

	for (i = 0; i < at_exit_nr_of_functions; i++)
		if (at_exit_functions[i])
			at_exit_functions[i]();
}

void exit(int code)
{
	atexit_run_functions();

	if (!l4ts_connected()) {
		LOG_printf("SIMPLE_TS not found -- cannot send exit event");
		l4_sleep_forever();
	}

	l4ts_exit();

	LOG_printf("Still alive, going zombie...\n");
	l4_sleep_forever();
}

#if USE_UCLIBC == 1
int __cxa_atexit(void (*function)(void));
int __cxa_atexit(void (*function)(void))
{
	return atexit(function);
}
#endif

/* -------------------------------------------------- */

void l4x_setup_threads(void)
{
	l4lx_thread_prio_set(linux_server_thread_id, CONFIG_L4_PRIO_SERVER);

	/* init task management subsystem */
	l4lx_task_init();	/* do this after rmgr_init() if used */
}

/* -------------------------------------------------- */
/* some irq stuff */

#if defined(CONFIG_L4_USERPRIV_ONLY) || defined(ARCH_arm)
#include <asm/hardirq.h>

void l4x_local_irq_disable(void)
{
	l4x_real_irq_disable();
	l4x_irq_flag(smp_processor_id()) = L4_IRQ_DISABLED;
}
EXPORT_SYMBOL(l4x_local_irq_disable);

void l4x_local_irq_enable(void)
{
	l4x_irq_flag(smp_processor_id()) = L4_IRQ_ENABLED;
	l4x_real_irq_enable();
}
EXPORT_SYMBOL(l4x_local_irq_enable);

unsigned long l4x_local_save_flags(void)
{
	return l4x_irq_flag(smp_processor_id());
}
EXPORT_SYMBOL(l4x_local_save_flags);

void l4x_local_irq_restore(unsigned long flags)
{
	l4x_irq_flag(smp_processor_id()) = flags;
	if (flags == L4_IRQ_ENABLED)
		l4x_real_irq_enable();

}
EXPORT_SYMBOL(l4x_local_irq_restore);
#endif

/* ----------------------------------------------------- */

int l4x_peek_upage(unsigned long addr,
                   unsigned long __user *datap,
                   int *ret)
{
	unsigned long tmp;

	if (addr >= UPAGE_USER_ADDRESS
	    && addr < UPAGE_USER_ADDRESS + PAGE_SIZE) {
		addr -= UPAGE_USER_ADDRESS;
		tmp = *(unsigned long *)(addr + &_upage_start);
		*ret = put_user(tmp, datap);
		return 1;
	}
	return 0;
}

/* ----------------------------------------------------- */

void l4x_printk_func(char *buf, int len)
{
	outnstring(buf, len);
}

#include <linux/hardirq.h>

/* ----------------------------------------------------- */
/* Prepare a thread for use as an IRQ thread. */
void l4x_prepare_irq_thread(struct thread_info *ti)
{
	/* Stack setup */
	*ti = (struct thread_info) INIT_THREAD_INFO(init_task);

	l4x_stack_setup(ti);

	ti->preempt_count = HARDIRQ_OFFSET;
	ti->addr_limit    = KERNEL_DS;

	/* Set pager */
	l4lx_thread_set_kernel_pager(l4_myself());
}

/* ----------------------------------------------------- */
void l4x_show_process(task_t *t)
{
#ifdef ARCH_x86
	printk("%2d: %s tsk st: %lx thrd flgs: %lx " PRINTF_L4TASK_FORM " esp: %08lx\n",
	       t->pid, t->comm, t->state, t->thread_info->flags,
	       PRINTF_L4TASK_ARG(t->thread.user_thread_id),
	       t->thread.kernel_sp);
#endif

#ifdef ARCH_arm
	printk("%2d: %s tsk st: %lx thrd flgs: %lx " PRINTF_L4TASK_FORM " esp: %08x\n",
	       t->pid, t->comm, t->state, t->thread_info->flags,
	       PRINTF_L4TASK_ARG(t->thread.user_thread_id),
	       t->thread_info->cpu_context.sp);
#endif
}

void l4x_show_processes(void)
{
	task_t *t;
	for_each_process(t) {
		if (t->pid >= 10)
			l4x_show_process(t);
	}
	printk("c");
	l4x_show_process(current);
}

void l4x_show_sigpending_processes(void)
{
	task_t *t;
	printk("Processes with pending signals:\n");
	for_each_process(t) {
		if (signal_pending(t))
			l4x_show_process(t);
	}
	printk("Signal list done.\n");

}

/* Just a function we can call without defining the header files */
void kdb_ke(void)
{
	enter_kdebug("kdb_ke");
}

#ifdef CONFIG_L4_DEBUG_SEGFAULTS
void l4x_print_vm_area_maps(task_t *p)
{
	struct vm_area_struct *vma = p->mm->mmap;


	while (vma) {
		struct file *file = vma->vm_file;

		LOG_printf("%p - %p", (void *)vma->vm_start, (void *)vma->vm_end);

		if (file) {
			char buf[40];
			int count = 0;

			char *s = buf;
			char *p = d_path(file->f_dentry, file->f_vfsmnt, s, sizeof(buf));
			if (!IS_ERR(p)) {
				while (s <= p) {
					char c = *p++;
					if (!c) {
						p = buf + count;
						count = s - buf;
						buf[count] = 0;
						break;
					} else if (!strchr("", c)) {
						*s++ = c;
					} else if (s + 4 > p) {
						break;
					} else {
						*s++ = '\\';
						*s++ = '0' + ((c & 0300) >> 6);
						*s++ = '0' + ((c & 070) >> 3);
						*s++ = '0' + (c & 07);
					}
				}
			}
			LOG_printf(" %s", count ? buf : "Unknown");
		}

		LOG_printf("\n");
		vma = vma->vm_next;
	}
}
#endif

#ifdef ARCH_x86
void sort_extable(struct exception_table_entry *start,
                  struct exception_table_entry *finish)
{
	/* We do not use the exception tables so do not sort it
	 * and _do_ _not_ _write_ _around_ _in_ _the_ _text_ _section_!
	 * (which is read-only!)
	 */
}
#endif

/* Some exports from L4 libraries etc. */
EXPORT_SYMBOL(LOG_printf);

#ifdef ARCH_x86

/* ----------------------------------------------------------------------- */
/* Export list, we could also put these in a separate file (like l4_ksyms.c) */

#include <linux/dma-mapping.h>
#include <linux/interrupt.h>

#include <asm/i387.h>
#include <asm/checksum.h>
#include <asm/delay.h>
#include <asm/dma-mapping.h>
#include <asm/io.h>

EXPORT_SYMBOL(l4env_vmalloc_memory_start);
EXPORT_SYMBOL(__VMALLOC_RESERVE);

EXPORT_SYMBOL(swapper_pg_dir);
EXPORT_SYMBOL(kernel_thread);
EXPORT_SYMBOL(__down_failed);
EXPORT_SYMBOL(__down_failed_interruptible);
EXPORT_SYMBOL(__down_failed_trylock);
EXPORT_SYMBOL(__up_wakeup);
EXPORT_SYMBOL(init_thread_union);
EXPORT_SYMBOL(csum_partial);
EXPORT_SYMBOL(csum_partial_copy);

EXPORT_SYMBOL(strstr);

#ifdef CONFIG_X86_USE_3DNOW
EXPORT_SYMBOL(_mmx_memcpy);
EXPORT_SYMBOL(mmx_clear_page);
EXPORT_SYMBOL(mmx_copy_page);
#endif


/* Exports for L4 specific modules */
EXPORT_SYMBOL(l4_sleep);
EXPORT_SYMBOL(l4rm_do_attach);
EXPORT_SYMBOL(l4rm_detach);

#include <l4/names/libnames.h>
EXPORT_SYMBOL(names_waitfor_name);

char l4env_ipc_errstrings[0];
EXPORT_SYMBOL(l4env_ipc_errstrings);

#endif /* ARCH_x86 */
